#!/usr/bin/env python
import argparse
import os
import numpy as np
import matplotlib.pyplot as plt

# 図の体裁
plt.style.use('classic')
plt_dic = {}
plt_dic['legend.fancybox'] = True
plt_dic['legend.labelspacing'] = 0.3
plt_dic['legend.numpoints'] = 3
plt_dic['figure.figsize'] = [8, 6]
plt_dic['axes.grid'] = True
plt_dic['font.size'] = 12
plt_dic['legend.fontsize'] = 12
plt_dic['axes.labelsize'] = 14
plt_dic['xtick.major.size'] = 5
plt_dic['xtick.minor.size'] = 3
plt_dic['xtick.direction'] = 'in'
plt_dic['savefig.bbox'] = 'tight'
plt_dic['savefig.dpi'] = 150
plt_dic['savefig.transparent'] = True
plt.rcParams.update(plt_dic)

description = """int file generater from dat."""
par = argparse.ArgumentParser(description=description)
par.add_argument('datfile')
par.add_argument('-o', '--outfile', required=True)
par.add_argument('-p,', '--plot', default=False,
                 action='store_true', help='Display data')
args = par.parse_args()


if args.outfile is None:
    base, ext = os.path.splitext(args.datfile)
    outfile = base + ".int"
else:
    outfile = args.outfile

o = open(outfile, "w")
data = np.loadtxt(args.datfile)
data[:, 1] = data[:, 1] / np.max(data[:, 1]) * 10000
o.write("GENERAL\n")
o.write(" %d\n" % (len(data)))
for d in data:
    o.write("%16.8f %20.8f\n" % (d[0], d[1]))
print(outfile, "was created.")

if args.plot is True:
    fig, ax = plt.subplots()
    ax.plot(data[:, 0], data[:, 1])
    plt.show()
    
    
