#!/usr/bin/env python
import re
import argparse

HEAD = """
#####################################################
! 測定方法 4-420のマックサイエンスの装置に設定している
#####################################################
# X線のデータ
NBEAM = 1
# 0:rietveld 1:simulation
NMODE = 0
# 出力の量
NPRINT = 1
# CuのK_alpha線
NTARG = 4
# (K_alpha2)/(K_alpha1)の強度比
R12 = 0.5
# 1.0:モノクロは使ってない, モノクロのブラッグ角
# CTHM1 = 0.7998
CTHM1 = 1.0
# 0:表面荒さの補正は行わない
NSURFR = 0
# 0: スリットは固定幅
NTRAN = 0
"""
phase_info = """
#####################################################
! PHASE情報
#####################################################
# 原子種のリスト (see RIETAN_VENUS/asfdc)
%s
PHASE{
PHNAME1 = '%s'
VNS1 = '%s'
HKLM1 = '%s'
"""

PHASE = """
#####################################################
! PHASEパラメータ
#####################################################
# 原子間距離の制限は課さない
LPAIR1 = 0
# 原子置換パラメータは自動設定
INDIV1 = 1
IHA1 = 0
IKA1 = 0
ILA1 = 1
IHP1 = 1
IKP1 = 0
ILP1 = 0
IHP2 = 0
IKP2 = 0
ILP2 = 0
IHP3 = 0
IKP3 = 0
ILP3 = 0
}
# ピークの線形(2:Pseudo-Voigt関数)
NPRFN = 2
# ピークのシフト補正(4:Legendre polynomials)
NSHIFT = 4
"""

phase_atom = """
#####################################################
# PHASE-A フィッテングパラメータの選択
#####################################################
PHASE-A{
# θのシフト
SHIFTN  0.0  0.0  0.0  0.0     0000
# 表面荒さの補正
ROUGH  0.0  0.0  0.0  0.0  0000
# バックグラウンド
BKGD  0.0  0.0  0.0  0.0  0.0  0.0
      0.0  0.0  0.0  0.0  0.0  0.0  000000000000
# スケール因子
SCALE  2.14608E-5  1
# ピークの線幅 (U, V, W, dummy)
FWHM12  2.45169E-2  -1.77615E-2  8.66975E-3  0.0  0000
# ピークの非対称性因子 (a0, a1, a2, dummy)
ASYM12  1.04564  0.14424  -4.14686E-2  0.0  0000
# ピークの減衰パラメータ (eta_L0, eta_L1, eta_H0, and eta_H1)
ETA12  0.58883  0.13955  0.502513  -4.12178E-2  0000
# ピークの非対称ブロードニングパラメータ, (Ue, Pe)
ANISOBR12  0.0  0.0  00
# ダミー
DUMMY12  0.0  0.0  0.0  0.0  0.0  0.0  0.0  0.0  0.0  0.0  0.0  0.0  0.0
         0.0  0.0  0.0  0000000000000000
# 配向性は考慮しない 
PREF  1.0  1.0  0.0  0.0  0.0  0.0  000000
# 格子定数 a, b, c, alpha, beta, gamma, 全原子の温度因子
CELLQ  %s
# ラベル, 占有率, 分率座標, 温度因子
%s}
# 最適化の制限
# {
# A(O1,y)=A(O1,x)
# }
"""

ANALYSIS = """
#####################################################
# 解析用のパラメータ
#####################################################
# MEM解析の時の分割数
NVOXA = %s
NVOXB = %s
NVOXC = %s
# 部分プロファイル緩和を適用する反射のプロファイル計算範囲はRIETANが決める.
NCUT = 0
#フィッティングする範囲を設定
# 0:全てのデータでfit  1:一部のデータでfit
NEXC = 0
If NEXC = 1 then
   # フィッテングしないθの範囲
   {
     0.0    14.99
     80.01  180.0
   }
end if
# 強度データのフォーマット
NINT =  1
NRANGE = 0

# フィッテング時のピークの裾の範囲 (+/-FWHM*PC)
PC = 40.00

#####################################################
# 最適化法のパラメータ
#####################################################
# Marquardt法
NLESQ = 0
# 標準偏差の評価法
NESU = 0
# 1つずつパラメータを最適化
NAUTO = 2
# 最適化サイクルの最大数
NCYCL = 100
# 収束チェックの閾値
CONV = 0.0001
# 収束チェックのサイクル数
NCONV = 6
# 0: パラメータの最適化に制限を課さない 1:課す
NC = 0
TK = 650.0
FINC = 2.0

*Update
# 1:insファイルをupdateする
NUPDT = 1
# 出力に関すること
*Graph
NPAT = 2
If NMODE <> 1 then
      IWIDTH = 800
      IHEIGHT = 400
      IYMIN = -2500
      IYMAX = 20000
      LBG = 0
      LDEL = 0
      IOFFSETD = -1500
      IPSIZE = 3
      IFSIZE = 12
      ILSIZE = 14
      INDREF = 0
      IOFFSET1 = -300
      /
else
      IWIDTH = 800
      IHEIGHT = 400
      LBG = 0
      IPSIZE =  3
      IFSIZE = 12
      ILSIZE = 14
end if

# Charge flippingのファイルを出力しない
NCF = 0
# ORFFE解析用のファイルを出力しない
NDA = 0
# Fourier/D synthesis用のファイルを出力しない
NFR = 0
# 0:MEM解析用のファイルを出力しない 1:する
NMEM = 0
If NMEM = 1 then
   LANOM = 0: 異常分散の寄与を取り除いた積分強度からσを計算する.
   LANOM = 1! 異常分散の寄与を含んだままの積分強度からσを計算する.

   LGR = 0: すべて独立な反射として出力する (LGR = 0に固定).
   LGR = 1! 重なった反射はグループ化して出力する.

   LFOFC = 0: リートベルト解析の結果に基づいて便宜上計算したFoを出力.
   LFOFC = 1! リートベルト解析における構造パラメーターから計算したFcを出力.

   EPSD = 0.001: グループ化する反射における面間隔の差の最大値 (Å).
   TSCAT1 = 500.0: 総電子数(X線回折)または正の干渉性散乱長の合計(中性子回折).
   TSCAT2 = 0.0: ゼロ(X線回折)または負の干渉性散乱長の合計(中性子回折).
end if

*Quit
"""

description = """rietan run program."""
par = argparse.ArgumentParser(description=description)
par.add_argument('insfile')
par.add_argument('-o', '--outfile', default=False, required=True,)
args = par.parse_args()

# title
body = open(args.insfile).read()
body = re.sub("\s*#.*", "", body)
body = re.sub("\s*!.*", "", body)
body = re.sub("^$", "", body, flags=re.M)
body = re.sub("^\s*\n", "", body)
title = body.split("\n")[0]

# phase_info
species = re.search("(\'\S+\'\s*)+\/", body).group(0)
vns = re.search("VNS1\s*=\s*\'(.+?)\'", body).group(1)
hklm = re.search("HKLM1\s*=\s*\'(.+?)\'", body).group(1)
phase_info = phase_info % (species, title, vns, hklm)

# phase_atom
data = re.findall(
    "^\s*(\S+\/\S+\s+([\+\-\.E0-9]+\s+){5}\s+[01]{5})", body, re.M)
atom = ""
for d in data:
    a = re.sub("((\S+\s+){5})\S+\s+(\S+)", "\\1 1.0   00000", d[0]) + "\n"
    # ラベルの1文字目は大文字
    a = a[0].upper() + a[1:]
    atom = atom + a
cellq = re.search("CELLQ\s+(.*)", body).group(1)
phase_atom = phase_atom % (cellq, atom)

# NVOXA, NVOXB, NVOXC
nvoxa = re.search("NVOXA\s*=\s*(\d+)", body).group(1)
nvoxb = re.search("NVOXB\s*=\s*(\d+)", body).group(1)
nvoxc = re.search("NVOXC\s*=\s*(\d+)", body).group(1)
ANALYSIS = ANALYSIS % (nvoxa, nvoxb, nvoxc)

o = open(args.outfile, "w")
o.write(title)
o.write(HEAD)
o.write(phase_info)
o.write(PHASE)
o.write(phase_atom)
o.write(ANALYSIS)
print(args.outfile, "was created.")
